@@ -69,4 +69,7 @@ apr_status_t h2_response_output_filter(ap_filter_t *f, apr_bucket_brigade *bb);
 
 apr_status_t h2_response_trailers_filter(ap_filter_t *f, apr_bucket_brigade *bb);
 
+void h2_from_h1_set_basic_http_header(apr_table_t *headers, request_rec *r,
+                                      apr_pool_t *pool);
+
 #endif /* defined(__mod_h2__h2_from_h1__) */
