@@ -86,7 +86,7 @@ const char *h2_h2_err_description(unsigned int h2_error)
     if (h2_error < (sizeof(h2_err_descr)/sizeof(h2_err_descr[0]))) {
         return h2_err_descr[h2_error];
     }
-    return "unknown http/2 errotr code";
+    return "unknown http/2 error code";
 }
 
 /*******************************************************************************
@@ -485,9 +485,9 @@ int h2_is_acceptable_connection(conn_rec *c, int require_all)
             if (strncmp("TLS", val, 3) 
                 || !strcmp("TLSv1", val) 
                 || !strcmp("TLSv1.1", val)) {
-            ap_log_cerror(APLOG_MARK, APLOG_DEBUG, 0, c, APLOGNO(03050)
-                          "h2_h2(%ld): tls protocol not suitable: %s", 
-                          (long)c->id, val);
+                ap_log_cerror(APLOG_MARK, APLOG_DEBUG, 0, c, APLOGNO(03050)
+                              "h2_h2(%ld): tls protocol not suitable: %s", 
+                              (long)c->id, val);
                 return 0;
             }
         }
@@ -680,7 +680,6 @@ static int h2_h2_post_read_req(request_rec *r)
         struct h2_task *task = h2_ctx_get_task(ctx);
         /* This hook will get called twice on internal redirects. Take care
          * that we manipulate filters only once. */
-        /* our slave connection? */
         if (task && !task->filters_set) {
             ap_filter_t *f;
             
